# What is Banshee? 
A high quality modern game development toolkit. It provides a **high performance multi-threaded game engine** written in C++14 that runs 2D and 3D games. The engine offers a wide variety of high level systems needed for game development, ranging from math and utility libraries, to DirectX 11 and OpenGL support all the way to input, GUI, physics, scripting and support for many popular resource formats (e.g. FBX, PNG, PSD, TTF).

On top of the engine Banshee also provides a highly intuitive and customizable **editor** that is easy to use by artists, designers and programmers alike. The editor allows you to manage all your project's assets, build levels, compile scripts, test and publish your game. Editor is also fully extensible via scripting to ensure developers can easily customize it for the exact needs of their projects.

The scripting system supports C# and comes with an extensive API ensuring you can complete your game without ever touching the C++ engine core. **C# scripting** makes your development easier by giving you access to the entire .NET library and a wide variety of pre-existing managed libraries. Integration of the scripting system with the editor and external tools like Visual Studio, as well as fast compilation times ensures that iteration times between coding and testing are minimized.

Aside from being a fully featured game engine and toolkit, Banshee also aims to provide a **high quality foundation**, so others may build and improve upon it. This is reflected in Banshee's architecture which is split into multiple layers allowing developers to use only the functionality they need, as well as its plugin oriented nature which allows developers to fully remove or replace major engine systems. You will find that Banshee's code is modern, with clean interfaces that make it easy to learn and maintain. Platform specific functionality is kept at a minimum making porting as easy as possible. Additionally its codebase is fully documented with an extensive API reference, as well as a set of manuals introducing you to most major systems.

# Features
* [Features](Documentation/GitHub/features.md) - A list of all currently available features.
* [Roadmap](Documentation/GitHub/roadmap.md) - A list of features to be implemented in both near and far future.

# Download
* [Binaries](Documentation/GitHub/install.md) - Download pre-built binaries for the most recent version.
* [Compiling](Documentation/GitHub/compiling.md) - Learn how to compile the engine yourself.

# Documentation
* [Managed](http://docs.banshee3d.com/Managed/index.html) - Documentation for the scripting (C#) API. This is what most users will be interested in.
* [Native] (http://docs.banshee3d.com/Native/index.html) - Documentation for the native (C++) API. This is meant for advanced users meaning to extend/modify the engine.
* [License](Documentation/GitHub/license.md) - Information about Banshee's license.

# Media
**Banshee Editor**
![Banshee Editor](http://bearishsun.thalassa.feralhosting.com/BansheeEditor.png "Banshee Editor")

**Video overview**
[Link] (https://youtu.be/WJsYOyCXGEU)

# Author
Banshee was created and is developed by Marko Pintera. I'm happy to connect with other developers, feel free to contact me at marko.pintera@gmail.com or add me on [LinkedIn] (https://goo.gl/t6pPPs). 